function lymelib2db(files, db, name)
%LYMELIB2DB   convert LyME library files to a LyME database file
%  lymelib2db(files, db) converts LyME library files whose paths
%  are given in list (or cell array) files to a database .PDB file.
%  lymelib2db(files, db, name) specifies the database name.

% Copyright 2001-2005, Calerga Sarl
% http://www.calerga.com

if nargin < 3
	name = 'LyMELibDB';
end
type = 'Lml ';
creator = 'LyME';

n = length(files);
lengths = zeros(1, n);
for i = 1:n
	libname = files{i};
	fd = fopen(libname, 'rt');
	data = fread(fd);
	fclose(fd);
	if any(libname == '.')
		libname(min(find(libname == '.')):end) = [];
	end
	if any(libname == ':' | libname == '/' | libname == 92)
		libname(1:max(find(libname == ':' | libname == '/' | libname == 92))) = [];
	end
	lengths(i) = length(libname) + 1 + length(data);
end

h = makePDBHeader(name, type, creator, lengths);

fdd = fopen(db, 'w');
fwrite(fdd, h, 'uint8');
for i = 1:n
	libname = files{i};
	fd = fopen(libname, 'rt');
	data = fread(fd);
	fclose(fd);
	if any(libname == '.')
		libname(min(find(libname == '.')):end) = [];
	end
	if any(libname == ':' | libname == '/' | libname == 92)
		libname(1:max(find(libname == ':' | libname == '/' | libname == 92))) = [];
	end
	fwrite(fdd, [libname(:); 10], 'uint8');
	fwrite(fdd, data, 'uint8');
end
fclose(fdd);


function h = makePDBHeader(name, type, creator, lengths)
%makePDBHeader   create a complete header recors of the specified length

rand('seed', clock * (1:6)');

name(end+1:32) = 0;
type(end+1:4) = ' ';
creator(end+1:4) = ' ';

dateb = bitshift(palmTime(clock), (-24:8:0)', 8);
lengthb = bitshift(length(lengths), [-8; 0], 8);

uniqueIDSeed = floor(255.99 * rand(1, 4));

h = [name(:)			% name
		0; 8			% attributes (backup)
		0; 0			% version
		dateb			% creation date
		dateb			% last modification date
		0; 0; 0; 0		% last backup date (none)
		0; 0; 0; 1		% modification number
		0; 0; 0; 0		% appInfoId (none)
		0; 0; 0; 0 		% sortInfoId (none)
		type(:)			% type
		creator(:)		% creator
		uniqueIDSeed(:)	% uniqueIDSeed
		0; 0; 0; 0		% nextRecordListID,
		lengthb];

offset = length(h) + 8 * length(lengths);
recList = zeros(8 * length(lengths), 1);
for i = 1:length(lengths)
	recList(8*i-7:8*i-4) = bitshift(offset, (-24:8:0)', 8);	% localChunkID
	offset = offset + lengths(i);
	recList(8*i-3) = 64;	% attributes = dirty
	recList(8*i-2:8*i) = bitshift(i, (-16:8:0)', 8);	% uniqueID
end

h = [h; recList];


function pt = palmTime(t)
%palmTime   conversion from [y,m,d,h,m,s] to Palm time

t(1) = t(1) - 1904;
t(3) = t(3) - 1;
m = cumsum([0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30]);
pt = t * [365*24*3600; 0; 24*3600; 3600; 60; 1] ...
		+ 24*3600 * (m(t(2)) ...
				+ ceil((t(1)+(t(2) >= 3))/4));
%pt = t * [365*24*3600; 0; 24*3600; 3600; 60; 1] ...
%		+ 24*3600 * (m(t(2)) ...
%				+ (t(2) >= 3 ? ceil((t(1)+1)/4) : ceil(t(1)/4)));
